function I_recon = admmnlm(b, lambda, rho, numIterADMM)
% Image Denoising using ADMM with NLM prior on greyscale image
% Kao Kitichotkul, 2020
% 
% This code is modified from Homework 6 from EE 367 class
% offered in Winter 2020 at Stanford University.
% The NLM implementation is obtained from
% https://www.mathworks.com/matlabcentral/fileexchange/27395-fast-non-local-means-1d-2d-color-and-3d
%
% Input
% -----
% b:            noisy image
% lambda:       lambda parameter for ADMM
% rho:          rho parameter for ADMM
% numIterADMM:  number of iterations for ADMM
%
% Output
% ------
% I_recon: reconstructed image

% When using for the first time, extract toolbox_nlmeans_version2.zip
% and run the following code to compile NLM function C code
% mex -v -compatibleArrayDims vectors_nlmeans_single.c 
% mex -v -compatibleArrayDims image2vectors_single.c 
% mex -v -compatibleArrayDims vectors_nlmeans_double.c 
% mex -v -compatibleArrayDims image2vectors_double.c

imSize = size(b);

% Perform ADMM with TV prior

% Parameters
kappa = lambda / rho;   % for z-update

% Initialize variables
x = zeros(imSize);
z = zeros(imSize);
u = zeros(imSize);

for iter = 1 : numIterADMM
    % x update
    v = z - u;
    x = (1 / (rho + 1)) * (b + rho * v);
    
    % z update
    v = x + u;
    Options.filterstrength=sqrt(kappa);
    z = NLMF(v, Options);
    
    % u update
    u = u + x - z;
end

I_recon = x;

end

