function I_recon = admmtv(b, lambda, rho, numIterADMM)
% Image Denoising using ADMM with TV prior on greyscale image
% Kao Kitichotkul, 2020
% 
% This code is modified from Homework 6 from EE 367 class
% offered in Winter 2020 at Stanford University.
%
% Input
% -----
% b:            noisy image
% lambda:       lambda parameter for ADMM
% rho:          rho parameter for ADMM
% numIterADMM:  number of iterations for ADMM
%
% Output
% ------
% I_recon: reconstructed image

imSize = size(b);

%%% Precompute terms for ADMM with TV prior

% noisy image in Fourier domain
bFT = fft2(b);

% convolution kernel for Dx and Dy
dx = [-1, 1];
dy = [-1; 1];

% precompute OTFs
p2o = @(x) psf2otf(x, imSize);
dxFT    = p2o(dx);          % Dx 
dxTFT   = conj(p2o(dx));    % DxT
dyFT    = p2o(dy);          % Dy
dyTFT   = conj(p2o(dy));    % DyT

% Function handle for computing gradient
Dfun = @(x) cat(3, real(ifft2(fft2(x).*dxFT)), real(ifft2(fft2(x).*dyFT)));

%%% Perform ADMM with TV prior

% Parameters
kappa = lambda / rho;   % for z-update

% Initialize variables
x = zeros(imSize);
z = cat(3, zeros(imSize), zeros(imSize));
u = cat(3, zeros(imSize), zeros(imSize));

% Denominator in x update
denom_x = p2o(1) + rho * (dxTFT .* dxFT + dyTFT .* dyFT);

for iter = 1 : numIterADMM
    % x update
    v = z - u;
    vFT = fft2(v);
    numer_x = bFT + rho * (dxTFT .* vFT(:, :, 1) + dyTFT .* vFT(:, :, 2));
    x = real(ifft2(numer_x ./ denom_x));
    
    % z update
    v = Dfun(x) + u;
    z = max(1 - kappa/abs(v), 0) .* v;
    
    % u update
    u = u + Dfun(x) - z;
end

I_recon = x;


end

