% Experiment on the Gaussian noise estimation algorithm

I = im2double(imread('images/einstein.tif'));   % ground truth image
imSize = size(I);
noise_levels = [0.001 0.005 0.01 0.05 0.1];
patch_sizes = [8 16];

noise_est_results = zeros(size(noise_levels, 2), size(patch_sizes, 2));

for id = 1:size(patch_sizes, 2)
    d = patch_sizes(id);
    for i = 1:size(noise_levels, 2)
        sigma = noise_levels(i);
        I_corr = I + sigma * randn(imSize);
        noise_est_results(i, id) = noise_estimate(I_corr, d);
    end
end

%% Plot results

figure
semilogx(noise_levels, noise_levels, 'o', 'displayname', 'injected noise level')
hold on; grid on
semilogx(noise_levels, noise_est_results(:, 1), 'o', 'displayname', 'estimate, d = 8')
semilogx(noise_levels, noise_est_results(:, 2), 'o', 'displayname', 'estimate, d = 16')
xlabel('Standard deviation of injected noise')
ylabel('Estimated standard deviation of the noise')
title('Estimation of Gaussian noise')
legend
saveas(gcf, 'noise_est_exp.png')