function xnorm = rl(b, nPhotons, numIter)
% Poisson Denoising using Richardson-Lucy algorithm for greyscale image
% Kao Kitichotkul, 2020
% 
% This code is modified from Homework 6 from EE 367 class
% offered in Winter 2020 at Stanford University.
% 
% Input
% -----
% b:        Poisson-noise corrupted image, not scaled by the 
%           number of photons (pixel values in [0, 1] scale)
% nPhotons: average number of photons on each pixel 
%           (variance of Poisson noise)   
% numIter:  number of iterations
% 
% Output
% ------
% xnorm: reconstructed image

% Scale the image to number of photons
b = b * nPhotons;

imageSize = size(b);

otf = psf2otf(1, imageSize);

% define function handle for 2D convolution
Afun    = @(x) ifft2( fft2(x).*otf, 'symmetric');
Atfun   = @(x) ifft2( fft2(x).*conj(otf), 'symmetric');

% Initial guess of the reconstructed image
x = rand(imageSize);

% Precalculation
denom = Atfun(ones(imageSize));                         % At1 term

for iter = 1 : numIter
    % Update the image
    enum = b ./ Afun(x);            % b/Ax
    enum(isnan(enum)) = 0;          % Set nan to 0
    enum = Atfun(enum);             % At(b/Ax)
    x = (enum ./ denom) .* x;       % At(b/Ax)/At1 .* x
end

% Normalize x
xnorm = x / nPhotons;

end

