% Experiment on setting threshold values for SURE-LET with soft threshold

%% Load image and create a noisy sample

I = im2double(imread('images/einstein.tif'));      % original image
imSize = size(I);
transformSize = imSize;

sigma = 0.1;                            % standard deviation of the white noise
I_corr = I + sigma * randn(imSize);     % noisy image

%% Try ranges of thresholds

T_test = [0 0.0001 0.0005 0.001 0.005 0.01 0.05 0.1 0.5 1 5 10 50 100];
numTbegin = 3;
numIter = size(T_test, 2) - numTbegin + 1;
MSE = zeros(1, numIter);
I_recon = zeros(imSize(1), imSize(2), numIter);

for i = 1 : numIter
   T = T_test(1:numTbegin - 1 + i);
   disp(['threshold values ' num2str(T)])
   I_recon(:, :, i) = softsure(I_corr, T, sigma);
   MSE(i) = (1/(imSize(1) * imSize(2))) * sum((I - I_recon(:, :, i)) .^ 2, 'all');
   disp(['MSE ' num2str(MSE(i))])
end

figure;
plot(numTbegin:size(T_test, 2), MSE, 'o')
grid on
xlabel('Range of thresholds of selected values')
ylabel('Mean squared error')
title(['Soft SURE-LET with a selection of threshold ranges, \sigma = ' num2str(sigma)])
saveas(gcf, ['softsure_exp1_sig' num2str(sigma) '.png'])

%% Fixed range of thresholds, range K = 3

T_test = [0 0.0001 0.0005 0.001 0.005 0.01 0.05 0.1 0.5 1 5 10 50 100];
K = 3;
numIter = size(T_test, 2) - K + 1;
MSE_fixed = zeros(1, numIter);
I_recon_fixed = zeros(imSize(1), imSize(2), numIter);

for i = 1 : numIter
   T = T_test(i:i + K - 1);
   disp(['threshold values ' num2str(T)])
   I_recon_fixed(:, :, i) = softsure(I_corr, T, sigma);
   MSE_fixed(i) = (1/(imSize(1) * imSize(2))) * sum((I - I_recon_fixed(:, :, i)) .^ 2, 'all');
   disp(['MSE ' num2str(MSE_fixed(i))])
end

figure;
plot(MSE_fixed, 'o')
grid on
xlabel('Sets of threshold values')
ylabel('Mean squared error')
title(['Soft SURE-LET with a selection of threshold values, threshold range = 3, \sigma = ' num2str(sigma)])
saveas(gcf, ['softsure_exp2_sig' num2str(sigma) '.png'])

